<?php

App::uses('EvCheckoutAppController', 'EvCheckout.Controller');
App::uses('ArrayUtil', 'EvCore.Lib');

class OrdersController extends EvCheckoutAppController {

	public function beforeFilter() {
		$this->adminActions[] = 'admin_pay';
		$this->adminActions[] = 'admin_status';
		$this->adminActions[] = 'admin_invoice';
		$this->adminActions[] = 'admin_notes';

		parent::beforeFilter();

		$this->Auth->allow(array('complete', 'cancel'));

		// specify whether order items can have their own, individual,
		// order item statuses
		$this->hasOrderItemStatuses = Configure::read('EvCheckout.allowOrderItemStatuses');
	}

	/**
	 * complete an order
	 * this could display success or fail messages.
	 * This is where the payment is checked and turned into an order
	 *
	 */
	public function complete() {
		if (CakePlugin::loaded('EvTransactions') && ! empty($this->request->query['transaction'])) {
			$this->TransactionsManager = $this->loadComponent('EvTransactions.Transactions');
			$this->OrderManager = $this->loadComponent('EvCheckout.OrderManager');

			if ($this->TransactionsManager->checkPayment()) {

				// Before we build the order we need to see if this transaction id refers to an already successful order,
				// in which case we wont want to build a new order
				$orderId = $this->_transactionAlreadyProcessed($this->request->query['transaction']);
				if (!$orderId) {
					// build the order. OrderBuilder is specified within Config
					// builds from an EvBasket Row as default
					$orderId = $this->OrderManager->buildOrder($this->request->query['transaction']);

					// trigger the success hook
					$this->getEventManager()->dispatch(
						new CakeEvent('EvCheckout.Controller.Order.success', $this, array(
							'orderId' => $orderId
						))
					);
				}

				$this->view = 'EvCheckout./Fallbacks/Order/success';
				$pageId = Configure::read('EvCheckout.pageIds.success');
			} else {

				// trigger the fail hook
				$this->getEventManager()->dispatch(
					new CakeEvent('EvCheckout.Controller.Order.fail', $this)
				);

				if (!empty(Configure::read('EvCheckout.failedOrder.redirect'))) {
					if (!empty(Configure::read('EvCheckout.failedOrder.message'))) {
						$this->Flash->fail(Configure::read('EvCheckout.failedOrder.message'));
					} else {
						$this->Flash->fail('Payment failed. Please check your card details and try again.');
					}

					$this->redirect(Configure::read('EvCheckout.failedOrder.redirect'));

				}

				$this->view = 'EvCheckout./Fallbacks/Order/fail';
				$pageId = Configure::read('EvCheckout.pageIds.fail');
			}
		}

		if (empty($this->view)) {
			// trigger the fail hook
			$this->getEventManager()->dispatch(
				new CakeEvent('EvCheckout.Controller.Order.fail', $this)
			);

			$this->view = 'EvCheckout./Fallbacks/Order/fail';
			$pageId = Configure::read('EvCheckout.pageIds.fail');
		}

		// check if we have a specific content page set
		// if so - set the template
		if (! empty($pageId)) {
			$pageData = $this->assignPage($pageId);

			if (! empty($pageData)) {
				if (! empty($orderId)) {
					$pageData['Page']['title'] = str_replace('ORDERID', $orderId, $pageData['Page']['title']);
					$pageData['Page']['body'] = str_replace('ORDERID', $orderId, $pageData['Page']['body']);
					$this->set('data', $pageData);
				}

				$this->view = $this->Tpl->getTemplate($pageData, 'Page');
			}
		}
	}

	/**
	 * cancel an order.
	 * usually used on gateways for the "cancel and return to store" links
	 *
	 */
	public function cancel() {
		if (CakePlugin::loaded('EvTransactions') && ! empty($this->request->query['transaction'])) {
			$this->TransactionsManager = $this->loadComponent('EvTransactions.Transactions');

			/*$this->TransactionsManager->failTransaction(
				$this->request->query['transaction'],
				'The order was cancelled by the user'
			);*/
		}

		// trigger the cancelled hook
		$this->getEventManager()->dispatch(
			new CakeEvent('EvCheckout.Controller.Order.cancel', $this)
		);

		$this->view = 'EvCheckout./Fallbacks/Order/cancel';
		$pageId = Configure::read('EvCheckout.pageIds.cancel');

		// check if we have a specific content page set
		// if so - set the template
		if (! empty($pageId)) {
			$pageData = $this->assignPage($pageId);

			if (! empty($pageData)) {
				$this->view = $this->Tpl->getTemplate($pageData, 'Page');
			}
		}
	}

/**
 * Checks to see if there is already a successful order for the given transaction id
 * This stops use creating duplicate orders for the same transaction if they user
 * refreshs the page etc
 * @param $transactionId The Transction ID you want to check
 * @return mixed The order id if there is one otherwise false
 */
	protected function _transactionAlreadyProcessed($transactionId) {
		$this->Transaction = EvClassRegistry::init('EvTransactions.Transaction');
		$OrderStatus = EvClassRegistry::init('EvCheckout.OrderStatus');

		$transaction = $this->Transaction->find('first', array(
				'conditions' => array(
					'Transaction.id' => $transactionId,
					'model' => 'EvCheckout.Order',
					'status' => 'success'
				),
				'contain' => array(
					'Order'
				)
			)
		);

		$unPaidStatusId = $OrderStatus->getConstant('UNPAID');

		if (!empty($transaction[$this->Transaction->alias]['model_id']) && ($transaction['Order']['order_status_id'] != $unPaidStatusId)) {
			return $transaction[$this->Transaction->alias]['model_id'];
		}

		return false;
	}

	/**
	 * redefine index columns
	 * to add more order info
	 */
	protected function _adminIndexColumns() {
		$columns = parent::_adminIndexColumns();

		$columns = ArrayUtil::addAfter(
			$columns,
			'Order.name',
			array(
				'Order.email' => array(
					'type' => 'string',
					'label' => __('Email')
				),
				'Order.phone' => array(
					'type' => 'string',
					'label' => __('Phone')
				),
				'OrderStatus.name' => array(
					'type' => 'string',
					'label' => __('Status')
				),
				'Order.is_paid' => array(
					'type' => 'display_boolean',
					'label' => __('Paid')
				),
				'Currency.name' => array(
					'type' => 'string',
					'label' => __('Currency')
				)
			)
		);

		unset($columns['Order.modified']);
		return $columns;
	}

	/*
		----------------------------------------------------------------
		Admin methods
		----------------------------------------------------------------
	*/

	/**
	 * redefine to remove the add button
	 *
	 */
	protected function _adminIndexToolbar() {
		return array();
	}

	/**
	 * redefine index paginate
	 * to contain status
	 */
	protected function _adminIndexPaginate() {
		$paginate = parent::_adminIndexPaginate();

		$paginate['contain'][] = 'OrderStatus';
		$paginate['contain'][] = 'Currency';
		$paginate['order'] = 'Order.id DESC';

		return $paginate;
	}

	/**
	 * Define an array of actions that can be performed on multiple items on the
	 * admin listing page.
	 *
	 * @return array
	 */
	protected function _adminMultiEditActions() {
		$actions = array('PrintInvoices' => 'Print Invoices');
		if (Configure::check('EvCheckout.dispatchedStatusId')) {
			$actions['MarkDispatched'] = 'Mark Dispatched';
		}
		return $actions;
	}

	/**
	 * define index filters
	 * to make it more usable
	 *
	 */
	protected function _adminFilterFields() {
		//Adding ID filter as first filter
		$fields = [
			'Order.ref' => [
				'label' => __('Id'),
				'type' => 'integer',
				'compare' => [
					'Order.id LIKE' => '%s'
				]
			]
		];

		$fields = array_merge($fields, parent::_adminFilterFields());

		$fields = ArrayUtil::addAfter(
			$fields,
			'Order.name',
			array(
				'Order.email' => array(
					'label' => __('Email'),
					'type' => 'string',
					'compare' => array(
						'Order.email LIKE' => '%%%s%%'
					)
				),
				'Order.order_status_id' => array(
					'label' => __('Status'),
					'type' => 'select',
					'compare' => array(
						'Order.order_status_id' => '%s'
					)
				),
				'Order.is_paid' => array(
					'label' => __('Paid'),
					'type' => 'select',
					'compare' => array(
						'Order.is_paid' => '%s'
					)
				),
				'Order.currency_id' => array(
					'label' => __('Currency'),
					'type' => 'select',
					'compare' => array(
						'Order.currency_id' => '%s'
					)
				)
			)
		);

		unset($fields['Order.modified']);

		return $fields;
	}

	/**
	 * redefine filter populate
	 * to bring in order status list
	 *
	 */
	protected function _adminPopulateFilterLookups() {
		$OrderStatus = EvClassRegistry::init('EvCheckout.OrderStatus');

		$orderStatuses = $OrderStatus->find('list', array(
			'conditions' => array(
				'is_selectable' => 1,
				'is_active' => 1
			)
		));

		$this->set(
			'orderStatuses',
			$orderStatuses
		);

		$this->set(
			'isPaids',
			array(
				'1' => __('Yes'),
				'0' => __('No')
			)
		);
	}

	/**
	 * redefine filter populate
	 * to bring in order status list
	 *
	 */
	protected function _adminPopulateLookups() {
		$OrderStatus = EvClassRegistry::init('EvCheckout.OrderStatus');

		$this->set(
			'orderStatuses',
			$OrderStatus->getForDropDown()
		);

		if (! empty($this->hasOrderItemStatuses)) {
			$OrderItemStatus = EvClassRegistry::init('EvCheckout.OrderItemStatus');
			$this->set(
				'orderItemStatuses',
				$OrderItemStatus->getForDropdown()
			);
		}
	}

	/**
	 * redefine to remove the add button
	 *
	 */
	protected function _adminFormToolbar($id = null) {
		$toolbar = parent::_adminFormToolbar($id);

		unset($toolbar['Add New']);

		$toolbar['<i class="fa fa-file"></i> Print Invoice'] = array(
			'url' => array(
				'admin' => true,
				'plugin' => 'ev_checkout',
				'controller' => 'orders',
				'action' => 'admin_invoice',
				$id
			),
			'class' => 'toolbar__btn',
			'target' => '_blank'
		);

		return $toolbar;
	}

	/**
	 * redefine the admin edit to have a nicer order management screen
	 *
	 * @param 	int 	$id 	The order ID
	 */
	public function admin_edit($id = null) {
		if ($id === null) {
			$this->Flash->fail(
				array(
					'title' => 'Error',
					'description' => 'No order was selected for viewing.'
				)
			);

			$this->redirect(
				array('action' => 'index')
			);
		}

		parent::admin_edit($id);

		// assign template variable to specify whether order items can have their
		// own individual statuses
		$this->set('hasOrderItemStatuses', $this->hasOrderItemStatuses);

		// If EvTranssaction is installed and we can find a transaction for this order, lets pull it in!
		if (CakePlugin::loaded('EvTransactions')) {
			$this->loadModel('EvTransactions.Transaction');
			$transactions = $this->Transaction->find('all', array(
				'conditions' => array(
					'OR' => array(
						array('model' => 'EvCheckout.Order'),
						array('model' => 'EvCheckoutOrder') // Gotta account for sites that override stuff!
					),
					'model_id' => $id
				)
			));

			if (! empty($transactions)) {
				$this->set(compact('transactions'));
			}
		}

		$this->view = 'EvCheckout.Orders/admin_edit';
	}

	/**
	 * save order notes
	 *
	 * @param  integer $id order id
	 * @return Redirect
	 */
	public function admin_notes($id) {
		$result = false;

		if ($this->request->is('post') || $this->request->is('put')) {
			$this->{$this->modelClass}->id = $id;

			$result = $this->{$this->modelClass}->save(array(
				'admin_notes' => $this->request->data['Order']['admin_notes'],
				'notes' => $this->request->data['Order']['notes']
			), array('deep' => true));

			if ($result) {
				$this->Flash->success(
					array(
						'title' => 'Notes Updated',
						'description' => 'The notes for this order have been updated.'
					)
				);
			}
		}

		if (! $result) {
			$this->Flash->fail(
				array(
					'title' => 'Failed',
					'description' => 'There was a problem updating your order notes, please try again.'
				)
			);
		}

		return $this->redirect(array(
			'plugin' => 'ev_checkout',
			'admin' => true,
			'controller' => 'orders',
			'action' => 'edit',
			$id
		));
	}

	/**
	 * mark an order as paid
	 *
	 * @param 	int 	$id 	the order ID to mark as paid
	 */
	public function admin_pay($id) {
		$this->OrderManager = $this->loadComponent('EvCheckout.OrderManager');

		if ($this->OrderManager->markAsPaid($id)) {
			$this->Flash->success(
				array(
					'title' => 'Paid',
					'description' => 'The order (#' . $id . ') has been marked as paid'
				)
			);
		} else {
			$this->Flash->fail(
				array(
					'title' => 'Failed',
					'description' => 'There was a problem marking the order (#' . $id . ') as paid'
				)
			);
		}

		$this->redirect(
			array(
				'action' => 'edit',
				$id
			)
		);
	}

	/**
	 * change an orders status
	 *
	 * @param 	int 	$id 	the order ID to change
	 */
	public function admin_status($id) {
		$this->OrderManager = $this->loadComponent('EvCheckout.OrderManager');

		$statusSet = ! empty($this->request->data['Order']['order_status_id']);

		if ($statusSet && $this->OrderManager->updateStatus($id, $this->request->data['Order']['order_status_id'])) {
			$this->Flash->success(
				array(
					'title' => 'Status Changed',
					'description' => 'The order status for #' . $id . ' has been updated'
				)
			);
		} else {
			$this->Flash->fail(
				array(
					'title' => 'Failed',
					'description' => 'There was a problem updating the status for order #' . $id
				)
			);
		}

		$this->redirect(
			array(
				'action' => 'edit',
				$id
			)
		);
	}

	/**
	 * view printable picking / invoice list
	 *
	 * @param 	int 	$id 	the order ID to view
	 */
	public function admin_invoice($id) {
		parent::admin_edit($id);

		$this->Meta->set(
			array(
				'Page' => array(
					'title' => 'Order #' . $id . ' Invoice',
				)
			),
			'EvCore.Page'
		);

		$currencyModel = EvClassRegistry::init('EvCurrency.Currency');
		$this->set('currencyList', $currencyModel->getForDropdown());

		$this->view = 'EvCheckout.Orders/admin_invoice';
		$this->layout = 'print';
	}

	/**
	 * frontend - show users order history
	 *
	 */
	public function history() {
		$this->OrderManager = $this->loadComponent('EvCheckout.OrderManager');

		$this->set(
			'orders',
			$this->OrderManager->getOrdersByUser($this->Auth->user('User.id'))
		);

		$this->Meta->set(
			array(
				'Page' => array(
					'title' => 'Order History'
				)
			),
			'EvCore.Page'
		);

		$this->view = 'EvCheckout.Orders/history';
	}

	/**
	 * frontend - show the user their order
	 *
	 * @param 	int 	$orderId 	The id of the order to view
	 */
	public function view($id) {
		$OrderModel = EvClassRegistry::init('EvCheckout.Order');
		$order = $OrderModel->readForView($id);

		if (! empty($order['Order']['user_id']) && (int)$order['Order']['user_id'] !== (int)$this->Auth->user('User.id')) {
			$route = $this->Routable->getItemRoute('EvCheckout', 'Order');
			$route['action'] = 'history';

			$this->Flash->fail('You do not have permission to view that order');

			$this->redirect($route);
		}

		$this->set(
			'order',
			$order
		);

		$this->Meta->set(
			array(
				'Page' => array(
					'title' => 'View Order #' . $id
				)
			),
			'EvCore.Page'
		);

		$this->view = 'EvCheckout.Orders/view';
	}

	/**
	 * Takes multiple orders and generates the invoices for them ready to print
	 * @return null
	 */
	protected function _processMultiEditPrintInvoices() {
		$Model = $this->{$this->modelClass};

		$orderIds = array_keys($this->request->data[$Model->alias]['id']);

		$orders = $Model->find('all', array(
			'contain' => array(
				'OrderItem' => array(
					'OrderItemData'
				),
				'OrderTotal' => array(
					'order' => 'OrderTotal.sequence ASC'
				),
				'OrderStatus',
				'OrderData'
			),
			'conditions' => array(
				'Order.id' => $orderIds
			)
		));

		$this->set('orders', $orders);

		$currencyModel = EvClassRegistry::init('EvCurrency.Currency');
		$this->set('currencyList', $currencyModel->getForDropdown());

		$this->layout = 'print';
		echo $this->render('EvCheckout.Orders/admin_bulk_invoice');
		die();
	}

/**
 * Takes multiple orders and updates their status to dispatched
 * @return null
 */
	protected function _processMultiEditMarkDispatched() {
		$Model = $this->{$this->modelClass};
		$orderIds = array_keys($this->request->data[$Model->alias]['id']);

		$this->OrderManager = $this->loadComponent('EvCheckout.OrderManager');

		foreach ($orderIds as $id) {
			$this->OrderManager->updateStatus($id, Configure::read('EvCheckout.dispatchedStatusId'));
		}
	}
}
